/*
 *  sudoku.cc
 *
 *  Written by
 *   Andreas Dehmel <zarquon@t-online.de>
 *
 *  Sudoku solver for NxN matrices. Input is a CSV-file containing the
 *  predefined values, with whitespace or 0 for values to calculate. The
 *  kind of matrix (3x3, 4x4, ...) is determined automatically from this
 *  input matrix.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 */


#include <new.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>



class SudokuIterator
{
  public:
  inline SudokuIterator(void) : _size(0), _r(0), _c(0) {}
  inline SudokuIterator(int size) : _size(size), _r(0), _c(0) {}
  inline SudokuIterator(const SudokuIterator &src) : _size(src._size), _r(src._r), _c(src._c) {}
  inline bool done(void) const {return (_r >= _size);}
  inline int row(void) const {return _r;}
  inline int col(void) const {return _c;}
  inline void reset(void) {_r = 0; _c = 0;}
  inline bool operator++(void) {
    if (_r < _size) {
      ++_c;
      if (_c >= _size) {_c = 0; ++_r;}
      return (_r < _size);
    }
    return false;
  }

  protected:
  int _size, _r, _c;
};



template<class T>
class SudokuData2D
{
  public:
  inline SudokuData2D(void) : _size(0), _data(0) {}
  inline SudokuData2D(int size) : _size(0), _data(0) {alloc(size);}
  virtual ~SudokuData2D(void);
  inline int size(void) const {return _size;}
  inline T &cell(int row, int col) {return _data[row*_size + col];}
  inline const T &cell(int row, int col) const {return _data[row*_size + col];}
  inline T &cell(const SudokuIterator &it) {return cell(it.row(), it.col());}
  inline const T &cell(const SudokuIterator &it) const {return cell(it.row(), it.col());}

  int  alloc(int size);
  void free(void);
  void clear(void);

  protected:
  int _size;
  T *_data;
};

template<class T>
SudokuData2D<T>::~SudokuData2D(void)
{
  free();
}

template<class T>
int SudokuData2D<T>::alloc(int size)
{
  free();
  _data = new(std::nothrow) T[size*size];
  if (_data != 0)
  {
    _size = size;
    return 0;
  }
  return -1;
}

template<class T>
void SudokuData2D<T>::free(void)
{
  if (_data != 0)
  {
    delete [] _data;
    _data = 0;
    _size = 0;
  }
}

template<class T>
void SudokuData2D<T>::clear(void)
{
  if (_data != 0)
    memset(_data, 0, _size*_size*sizeof(T));
}



class SudokuInteger2D : public SudokuData2D<int>
{
  public:
  inline SudokuInteger2D(void) {}
  inline SudokuInteger2D(int size) : SudokuData2D<int>(size) {}
  void print_formatted(int maxnum) const;
};

void SudokuInteger2D::print_formatted(int maxnum) const
{
  int i, dec, n;
  char format[8];
  for (i=1, dec=10; dec < maxnum; i++, dec *= 10) ;
  sprintf(format, ",%%%dd", i);
  for (i=0, n=size(); i<n; i++)
  {
    printf(format+1, cell(i, 0));
    for (int j=1; j<n; j++)
      printf(format, cell(i, j));
    printf("\n");
  }
}





class SudokuRawData : public SudokuInteger2D
{
  public:
  SudokuRawData(const char *file);
  SudokuRawData(int cdim);
  void print(void) const;
  inline int celldim(void) const {return _celldim;}

  protected:
  static int parse_line(const char *line, int *data, int size);
  int _celldim;
};


SudokuRawData::SudokuRawData(const char *file) : _celldim(0)
{
  FILE *fp = fopen(file, "r");
  if (fp != 0)
  {
    char buffer[256];
    int num;

    buffer[255] = '\0';
    fgets(buffer, 255, fp);
    num = parse_line(buffer, 0, INT_MAX);
    if (num > 0)
    {
      for (_celldim = 2; _celldim*_celldim < num; ++_celldim) ;

      if (alloc(_celldim*_celldim) == 0)
      {
        clear();

        int *dptr = _data;
        parse_line(buffer, dptr, _size);

        num = 1;
        while (!feof(fp) && (num < _size))
        {
          dptr += size();
          ++num;
          fgets(buffer, 255, fp);
          parse_line(buffer, dptr, _size);
        }
      }
    }
    fclose(fp);
  }
}

SudokuRawData::SudokuRawData(int cdim) : _celldim(0)
{
  if (alloc(cdim*cdim) == 0)
  {
    _celldim = cdim;
    clear();
  }
}

int SudokuRawData::parse_line(const char *buffer, int *data, int size)
{
  const char *b = buffer, *rest;
  int ctr = 0;

  while (*b != '\0')
  {
    int v = 0;

    while (isspace(*b)) ++b;
    if ((*b != '\0') && (*b != ','))
    {
      v = strtol(b, (char**)&rest, 10);
      if (b == rest)
        return -1;

      b = rest;
      while ((*b != '\0') && (*b != ',')) ++b;
    }

    if (*b == ',') ++b;

    if ((data != 0) && (ctr < size))
      data[ctr] = v;
    ctr++;
  }
  return ctr;
}

void SudokuRawData::print(void) const
{
  print_formatted(size());
}




// base class for reservoir bits
class SudokuBitsBase
{
  public:
  SudokuBitsBase(void) : _size(0) {}
  SudokuBitsBase(int size) : _size(size) {}
  void set_size(int size) {_size = size;}
  int  size(void) const {return _size;}

  protected:
  int _size;
};

// this implementation is limited to permutating numbers <= number of bits in a long
class SudokuBitsLong : public SudokuBitsBase
{
  public:
  inline SudokuBitsLong(void) : _bits(0) {}
  inline SudokuBitsLong(int size) : SudokuBitsBase(size), _bits(0) {}
  inline SudokuBitsLong(const SudokuBitsLong &b1, const SudokuBitsLong &b2) :
    SudokuBitsBase(b1._size), _bits(b1._bits | b2._bits) {}
  inline bool available(int n) const {return ((_bits & (1 << (n-1))) == 0);}
  inline void free(int n) {_bits &= ~(1 << (n-1));}
  inline void alloc(int n) {_bits |= (1 << (n-1));}
  inline void clear(void) {_bits = 0;}
  inline bool done(void) const {return (_bits+1 == static_cast<unsigned long>(1 << _size));}

  protected:
  unsigned long _bits;
};

// generic array-based implementation for arbitrary numbers
class SudokuBitsArray : public SudokuBitsBase
{
  public:
  inline SudokuBitsArray(void) : _bits(0) {}
  inline SudokuBitsArray(int size) : _bits(0) {set_size(size);}
  SudokuBitsArray(const SudokuBitsArray &b1, const SudokuBitsArray &b2);
  inline ~SudokuBitsArray(void) {flush();}
  inline bool available(int n) const {return ((_bits[(n-1)>>3] & (1 << ((n-1)&7))) == 0);}
  inline void free(int n) {_bits[(n-1)>>3] &= ~(1 << ((n-1)&7));}
  inline void alloc(int n) {_bits[(n-1)>>3] |= (1 << ((n-1)&7));}
  void set_size(int size);
  void clear(void);
  bool done(void) const;

  protected:
  inline int array_size(void) const {return (_size + 7) >> 3;}
  void flush(void);
  unsigned char *_bits;
};

SudokuBitsArray::SudokuBitsArray(const SudokuBitsArray &b1, const SudokuBitsArray &b2) : _bits(0)
{
  set_size(b1._size);
  if (_bits != 0)
  {
    for (int i=0, n=array_size(); i<n; i++)
      _bits[i] = b1._bits[i] | b2._bits[i];
  }
}

void SudokuBitsArray::flush(void)
{
  if (_bits != 0)
  {
    delete [] _bits;
    _bits = 0;
  }
}

void SudokuBitsArray::set_size(int size)
{
  SudokuBitsBase::set_size(size);
  flush();
  int n = array_size();
  _bits = new(std::nothrow) unsigned char[n];
  if (_bits != 0)
    memset(_bits, 0, n);
}

void SudokuBitsArray::clear(void)
{
  if (_bits != 0)
  {
    for (int i=0, n=array_size(); i<n; i++)
      _bits[i] = 0;
  }
}

bool SudokuBitsArray::done(void) const
{
  if (_bits != 0)
  {
    int i, n = array_size();
    for (i=0; i<n-1; i++)
    {
      if (_bits[i] != 0xff)
        return false;
    }
    unsigned char mask = 0xff >> ((8 - (_size & 7)) & 7);
    //for (i=0; i<n; i++) printf("%02x", _bits[i]); printf(" [%02x]\n", mask);
    return ((_bits[n-1] & mask) == mask);
  }
  return true;
}



// Template class for number reservoir
template<class Bits>
class SudokuReservoir
{
  public:
  inline SudokuReservoir(void) : _reservoir(0) {}
  inline SudokuReservoir(int size) : _reservoir(size) {}
  inline void set_size(int size) {_reservoir.set_size(size);}
  inline int  size(void) const {return _reservoir.size();}
  inline bool available(int n) const {return _reservoir.available(n);}
  inline void free(int n) {_reservoir.free(n);}
  inline void alloc(int n) {_reservoir.alloc(n);}
  inline void clear(void) {_reservoir.clear();}
  inline bool done(void) const {return _reservoir.done();}
  inline SudokuReservoir<Bits> merge(const SudokuReservoir<Bits> &r) const {
    return SudokuReservoir<Bits>(_reservoir, r._reservoir);
  }
  int  first(void) const;
  bool singular(int &v) const;
  void print(void) const;
  static inline bool legal_size(int size) {return (size <= static_cast<int>(8*sizeof(unsigned long)));}

  protected:
  inline SudokuReservoir(const Bits &b1, const Bits &b2) : _reservoir(b1, b2) {}
  Bits _reservoir;
};

template<class Bits>
int SudokuReservoir<Bits>::first(void) const
{
  for (int i=1, n=_reservoir.size(); i<=n; i++)
  {
    if (available(i))
      return i;
  }
  return -1;
}

template<class Bits>
bool SudokuReservoir<Bits>::singular(int &v) const
{
  int i, n;
  for (i=1, v=0, n=_reservoir.size(); i<=n; i++)
  {
    if (available(i))
    {
      v = i;
      break;
    }
  }
  for (++i; i<=n; i++)
  {
    if (available(i))
      return false;
  }
  return (v != 0);
}

template<class Bits>
void SudokuReservoir<Bits>::print(void) const
{
  for (int i=1, n=_reservoir.size(); i<=n; i++)
  {
    if (available(i))
      printf(" %d", i);
  }
}




// Template class for a macro cell (NxN integers); NxN macro cells form the full Sudoku matrix
template<class Reservoir>
class SudokuMacroCell : public SudokuInteger2D
{
  public:
  inline SudokuMacroCell(void) {}
  inline SudokuMacroCell(int row, int col, const SudokuRawData &data) {init(row, col, data);}
  inline SudokuReservoir<Reservoir> &reservoir(void) {return _reservoir;}
  inline const SudokuReservoir<Reservoir> &reservoir(void) const {return _reservoir;}
  int init(int row, int col, const SudokuRawData &data);
  int write(int row, int col, SudokuRawData &data) const;
  SudokuIterator first(void) const;
  void print(void) const;

  protected:
  SudokuReservoir<Reservoir> _reservoir;
};

template<class Reservoir>
int SudokuMacroCell<Reservoir>::init(int row, int col, const SudokuRawData &data)
{
  if (alloc(data.celldim()) == 0)
  {
    _reservoir.set_size(data.size());

    int r = row*_size, c = col*_size;
    for (int i=0; i<_size; i++)
    {
      for (int j=0; j<_size; j++)
      {
        int v = data.cell(r+i, c+j);
        cell(i, j) = v;
        if (v != 0)
          _reservoir.alloc(v);
      }
    }
    return 0;
  }
  return -1;
}

template<class Reservoir>
int SudokuMacroCell<Reservoir>::write(int row, int col, SudokuRawData &data) const
{
  if (_size == data.celldim())
  {
    int r = row*_size, c = col*_size;
    for (int i=0; i<_size; i++)
    {
      for (int j=0; j<_size; j++)
        data.cell(r+i, c+j) = cell(i, j);
    }
    return 0;
  }
  return -1;
}

template<class Reservoir>
SudokuIterator SudokuMacroCell<Reservoir>::first(void) const
{
  SudokuIterator it(_size);
  while (!it.done())
  {
    if (cell(it.row(), it.col()) == 0)
      break;
    ++it;
  }
  return it;
}

template<class Reservoir>
void SudokuMacroCell<Reservoir>::print(void) const
{
  print_formatted(_size*_size);
}





// Abstract base class for Sudoku matrices
class SudokuBaseMatrix
{
  public:
  virtual ~SudokuBaseMatrix(void) {}
  virtual bool calculate(void) = 0;
  virtual int  write(SudokuRawData &data) const = 0;
  virtual bool verify(void) const = 0;
};


// Template class for concrete Sudoku matrices
template<class Reservoir>
class SudokuMatrix : public SudokuBaseMatrix
{
  public:
  SudokuMatrix(const SudokuRawData &data);
  virtual ~SudokuMatrix(void);
  virtual bool calculate(void);
  virtual int  write(SudokuRawData &data) const;
  virtual bool verify(void) const;

  protected:
  void free(void);
  bool sudoku_test(const SudokuIterator &cit, const SudokuIterator &vit);
  bool sudoku_next(const SudokuIterator &cit, const SudokuIterator &vit);
  void preprocess(void);

  SudokuReservoir<Reservoir> *_rowRes;
  SudokuReservoir<Reservoir> *_colRes;
  SudokuData2D< SudokuMacroCell<Reservoir> > _mcells;
};

template<class Reservoir>
SudokuMatrix<Reservoir>::SudokuMatrix(const SudokuRawData &data) : _rowRes(0), _colRes(0)
{
  if (_mcells.alloc(data.celldim()) == 0)
  {
    int n = data.size();

    _rowRes = new(std::nothrow) SudokuReservoir<Reservoir>[n];
    _colRes = new(std::nothrow) SudokuReservoir<Reservoir>[n];

    if ((_rowRes != 0) && (_colRes != 0))
    {
      for (int i=0; i<n; i++)
      {
        _rowRes[i].set_size(n);
        _colRes[i].set_size(n);
      }

      for (int i=0, cd=_mcells.size(); i<cd; i++)
      {
        for (int j=0; j<cd; j++)
          _mcells.cell(i, j).init(i, j, data);
      }

      for (int i=0; i<n; i++)
      {
        for (int j=0; j<n; j++)
        {
          int v = data.cell(i, j);
          if (v != 0)
          {
            _rowRes[i].alloc(v);
            _colRes[j].alloc(v);
          }
        }
      }
    }
    else
    {
      free();
    }
  }
}

template<class Reservoir>
SudokuMatrix<Reservoir>::~SudokuMatrix(void)
{
  free();
}

template<class Reservoir>
void SudokuMatrix<Reservoir>::free(void)
{
  if (_rowRes != 0)
  {
    delete [] _rowRes;
    _rowRes = 0;
  }
  if (_colRes != 0)
  {
    delete [] _colRes;
    _colRes = 0;
  }
  _mcells.free();
}

template<class Reservoir>
int SudokuMatrix<Reservoir>::write(SudokuRawData &data) const
{
  int size = _mcells.size();
  if (size == data.celldim())
  {
    for (int i=0; i<size; i++)
    {
      for (int j=0; j<size; j++)
        _mcells.cell(i, j).write(i, j, data);
    }
    return 0;
  }
  return -1;
}

template<class Reservoir>
void SudokuMatrix<Reservoir>::preprocess(void)
{
  bool found = false;
  do
  {
    int size = _mcells.size();
    SudokuIterator cit(size);
    found = false;
    while (!cit.done())
    {
      SudokuMacroCell<Reservoir> &mc = _mcells.cell(cit);
      SudokuIterator vit(size);
      while (!vit.done())
      {
        if (mc.cell(vit) == 0)
        {
          int gr = cit.row()*size + vit.row(), gc = cit.col()*size + vit.col(), v;
          SudokuReservoir<Reservoir> r(mc.reservoir().merge(_rowRes[gr]).merge(_colRes[gc]));
          //printf("PP (%d,%d): ", gr, gc); r.print(); printf("\n");
          if (r.singular(v))
          {
            //printf("SINGULAR (%d,%d): %d\n", gr, gc, v);
            _rowRes[gr].alloc(v);
            _colRes[gc].alloc(v);
            mc.reservoir().alloc(v);
            mc.cell(vit) = v;
            found = true;
          }
        }
        ++vit;
      }
      ++cit;
    }
  }
  while (found);
}

template<class Reservoir>
bool SudokuMatrix<Reservoir>::calculate(void)
{
  preprocess();

  SudokuIterator cit(_mcells.size());
  SudokuIterator vit = _mcells.cell(cit).first();

  return sudoku_test(cit, vit);
}

template<class Reservoir>
bool SudokuMatrix<Reservoir>::sudoku_test(const SudokuIterator &cit, const SudokuIterator &vit)
{
  if (!vit.done())
  {
    SudokuMacroCell<Reservoir> &mc = _mcells.cell(cit);
    int size = _mcells.size(), gr = cit.row()*size + vit.row(), gc = cit.col()*size + vit.col();
    SudokuReservoir<Reservoir> r(mc.reservoir().merge(_rowRes[gr]).merge(_colRes[gc]));
    //printf("(%d,%d), (%d,%d): ", cit.row(), cit.col(), vit.row(), vit.col()); r.print(); printf("\n");
    for (int f = 1; f <= r.size(); f++)
    {
      if (r.available(f))
      {
        _rowRes[gr].alloc(f);
        _colRes[gc].alloc(f);
        mc.reservoir().alloc(f);
        mc.cell(vit) = f;
        //mc.print();

        if (sudoku_next(cit, vit))
          return true;

        _rowRes[gr].free(f);
        _colRes[gc].free(f);
        mc.reservoir().free(f);
        mc.cell(vit) = 0;
        //mc.print();
      }
    }
    return false;
  }
  return true;
}

template<class Reservoir>
bool SudokuMatrix<Reservoir>::sudoku_next(const SudokuIterator &cit, const SudokuIterator &vit)
{
  SudokuMacroCell<Reservoir> &mc = _mcells.cell(cit);
  SudokuIterator nextv(vit), nextc(cit);

  ++nextv;
  while (!nextv.done())
  {
    if (mc.cell(nextv) == 0)
      break;
    ++nextv;
  }

  while (nextv.done())
  {
    ++nextc;
    if (nextc.done())
      return true;

    nextv = _mcells.cell(nextc).first();
  }

  return sudoku_test(nextc, nextv);
}

template<class Reservoir>
bool SudokuMatrix<Reservoir>::verify(void) const
{
  bool status = true;
  int size = _mcells.size();

  // round 1: macrocells
  SudokuIterator cit(size);
  SudokuReservoir<Reservoir> r1(size*size), r2(size*size);
  while (!cit.done())
  {
    const SudokuMacroCell<Reservoir> &mc = _mcells.cell(cit);
    SudokuIterator vit(size);
    r1.clear();
    while (!vit.done())
    {
      int v = mc.cell(vit);
      if ((v == 0) || !r1.available(v))
        status = false;
      else
        r1.alloc(v);
      ++vit;
    }
    if (!r1.done())
      status = false;

    ++cit;
  }

  // round 2: rows/columns
  for (int i=0; i<size; i++)
  {
    for (int j=0; j<size; j++)
    {
      r1.clear();
      r2.clear();
      for (int ii=0; ii<size; ii++)
      {
        for (int jj=0; jj<size; jj++)
        {
          int v = _mcells.cell(i, ii).cell(j, jj);
          if ((v == 0) || !r1.available(v))
            status = false;
          else
            r1.alloc(v);

          v = _mcells.cell(ii, i).cell(jj, j);
          if ((v == 0) | !r2.available(v))
            status = false;
          else
            r2.alloc(v);
        }
      }
      if (!r1.done() || !r2.done())
        status = false;
    }
  }

  return status;
}




int main(int argc, const char *argv[])
{
  if (argc < 2)
  {
    printf("Usage: %s <csv-data>\n", argv[0]);
    return -1;
  }

  SudokuRawData data(argv[1]);
  //data.print();

  SudokuBaseMatrix *matrix = 0;

  // Try to use the more efficient long reservoir if possible, otherwise fall back on the array
  if (SudokuReservoir<SudokuBitsLong>::legal_size(data.size()))
    matrix = new SudokuMatrix<SudokuBitsLong>(data);
  else
    matrix = new SudokuMatrix<SudokuBitsArray>(data);

  matrix->calculate();

  if (matrix->verify())
  {
    SudokuRawData result(data.celldim());
    matrix->write(result);
    result.print();
  }
  else
  {
    fprintf(stderr, "Inconsistent result, inform author\n");
  }

  delete matrix;

  return 0;
}
